c  genveld.f
c  Generate a size-dependent isotropic velocity distribution.
c  Miroslav Broz (miroslav.broz@email.cz), Dec 7th 2010
c

c sigma = V_5km * (5 km/D)
c f(v_x) = exp(-(v_x/(2 sigma))^2) <- pro SLOZKY rychlosti!
c dN_x = f(v_x) dv_x
c rozdeleni VELIKOSTI rychlosti by melo odpovidat v^2 exp(-(v/(2 sigma))^2)

      program genveld

      implicit none

      integer nmax
      real*8 pi,G
      parameter(pi = 3.1415926535d0, G = 6.67e-11)
      parameter(nmax = 100000)

c input
      real*8 V_5km,D_5km,alpha,vmin,vmax,D_PB,rho
      integer idum
      character*80 infile

c temporary
      real*8 vej,vmean,p,vx,vy,vz,R,dNmax,dN,vesc,x,sigma
      real*8 D(nmax)
      real*8 nx,ny,nz
      integer n,i,ierr
      character*80 s

c  functions
      real*8 ran1,ran2
      integer length

      read(*,*) V_5km
      read(*,*) D_5km
      read(*,*) alpha
      read(*,*) vmin
      read(*,*) vmax
      read(*,*) D_PB
      read(*,*) rho
      read(*,*) idum
      read(*,*) infile

c print input parameters for reference
      n = int(x)
      D_PB = D_PB*1.d3
      write(*,30) '# V_5km = ',V_5km, ' m/s'
30    format(a,g12.6,a)
      write(*,30) '# D_5km = ',D_5km, ' km'
      write(*,30) '# alpha = ',alpha
      write(*,30) '# vmin = ',vmin, ' m/s'
      write(*,30) '# vmax = ',vmax, ' m/s'
      write(*,30) '# D_PB = ', D_PB, ' m'
      write(*,30) '# rho = ',rho, ' kg/m^3'
      write(*,40) '# idum = ',idum,''
40    format(a,i5,a)
      write(*,50) '# infile = ', infile(1:length(infile)), ''
50    format(a,a,a)

c calculate the escape velocity from the asteroid
      R = D_PB/2.d0
      vesc = sqrt(8./3.*pi*G*rho) * R

      write(*,30) '# vesc = ',vesc, ' m/s'

c read file with diameters
      open(unit=10,file=infile,iostat=ierr)
      if (ierr.ne.0) then
        write(*,*) 'Error opening file ', infile
        stop
      endif

      i = 1
60    continue
        read(10,*,end=70,err=70) D(i)
        i = i+1
        if (i.gt.nmax) then
          write(*,*) 'Error: too many diameters, i .gt. nmax = ', nmax
        endif
      goto 60
70    continue
      n = i-1
      write(*,40) '# n = ',n, ''
      write(*,40) '# vx vy vz D vej i'
      write(*,40) '# m/s m/s m/s m m/s 1'

      close(10)

c 1st body (largest remnant) remains fixed
c      write(*,20) 0.d0,0.d0,0.d0,D(1),0.d0,1

c 2nd body (largest fragment) remains fixed; use transversal velocity!
c      vej = 2.5d0
c      nx = 1.d0
c      ny = 1.d0
c      nz = 1.d0
c      call normalize(nx,ny,nz)
c      vx = nx*vej
c      vy = ny*vej
c      vz = nz*vej
c      write(*,20) vx,vy,vz,D(2),vej,2

c Monte-Carlo algorithm to generate distribution of v-COMPONENTS!
      vmean=0.d0
      dNmax = 1.d0    ! this is maximum of the exponential function dN()
      do i = 1, n
        sigma = V_5km * (D(i)/D_5km)**(alpha)

81      continue
          vx = ran2(vmin,vmax,idum)
          p = ran1(idum)
        if (p.gt.dN(vx,sigma)/dNmax) goto 81

82      continue
          vy = ran2(vmin,vmax,idum)
          p = ran1(idum)
        if (p.gt.dN(vy,sigma)/dNmax) goto 82

83      continue
          vz = ran2(vmin,vmax,idum)
          p = ran1(idum)
        if (p.gt.dN(vz,sigma)/dNmax) goto 83

        vej = sqrt(vx**2+vy**2+vz**2)
        write(*,*) vx,vy,vz,D(i),vej,i
20      format(3(1x,e22.16),1x,f6.2,2x,f6.2,1x,i5)

        vmean=vmean+vej
      enddo

c calculate mean velocity among fragments
      vmean = vmean/n
      write(*,30) '# vmean = ',vmean, ' m/s'

      stop
      end

c-----------------------------------------------------------------------
      real*8 function dN(v,sigma)
      real*8 v,sigma

      dN = exp(-v**2/(2.d0*sigma**2))

      return
      end

c-----------------------------------------------------------------------
      real*8 function ran2(vmin,vmax,idum)
      real*8 vmin,vmax
      integer idum
c  functions
      real*8 ran1,sgn,tmp

      tmp = (2.d0*ran1(idum)-1.d0)
      ran2 = vmin*sgn(tmp) + (vmax-vmin)*tmp

      return
      end

      subroutine normalize(x,y,z)
      real*8 x,y,z
      real*8 r
      r = sqrt(x*x+y*y+z*z)
      x = x/r
      y = y/r
      z = z/r
      return
      end


